//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * 基本フレームワークヘッダ
 * @author Junpee
 */

#ifndef BASIC_FRAMEWORK_H_
#define BASIC_FRAMEWORK_H_

#include <Framework/System/SimpleFramework.h>

#include <Graphics/System/GraphicsDeviceObjectHolder.h>
#include <Graphics/Enumeration/ConfirmGraphicsDevice.h>
#include <Core/Utility/Timer.h>


namespace Lamp{

class FPSMeasurement;

//------------------------------------------------------------------------------
/**
 * 基本フレームワーク
 *
 * シンプルフレームワークにID3DXによるフォント描画とFPS計測と
 * グラフィックスデバイス選択を追加したフレームワーク。
 */
class BasicFramework :
	public SimpleFramework, public GraphicsDeviceObjectHolder,
	public ConfirmGraphicsDevice{
public:
	//--------------------------------------------------------------------------
	// 生成、破棄
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 * @param name アプリケーション名
	 */
	BasicFramework(const String& name);

	/**
	 * デストラクタ
	 */
	virtual ~BasicFramework();

	//--------------------------------------------------------------------------
	// オーバーライドメソッド
	//--------------------------------------------------------------------------
	/**
	 * グラフィックスデバイスの確認
	 *
	 * デバイスの能力を確認して、使用するならtrue、使用しないならfalseを返す。
	 * @param deviceCapability デバイス能力
	 * @param vertexProcessingType 頂点プロセスタイプ
	 * @param adapterFormat アダプタのフォーマット
	 * @param backBufferFormat バックバッファのフォーマット
	 * @return デバイスの使用を許可するならばtrueを返す。
	 */
	virtual bool confirmGraphicsDevice(
		const D3DCapacity& deviceCapability, u_int vertexProcessingType,
		D3DFORMAT adapterFormat, D3DFORMAT backBufferFormat){
		// 全てのデバイスを許可する
		return true;
	}

	//--------------------------------------------------------------------------
	// デバイスオブジェクト関連
	//--------------------------------------------------------------------------
	/**
	 * デバイスオブジェクトの初期化
	 *
	 * デバイスオブジェクトの初期化はinitializeDeviceObjectsをオーバライドする
	 * @return 成功したらtrueを返す
	 */
	virtual bool initializeGraphicsDeviceObjects();

	/**
	 * デバイスオブジェクトの削除
	 *
	 * デバイスオブジェクトの削除はdeleteDeviceObjectsをオーバライドする
	 */
	virtual void deleteGraphicsDeviceObjects();

	/**
	 * デバイスオブジェクトのリストア
	 *
	 * デバイスオブジェクトのリストアはrestoreDeviceObjectsをオーバライドする
	 * このメソッド内ででデバイスステートを変更すると、デフォルトステートとなり、
	 * レンダラセットアップ時にそのステートが復元されます。
	 * @return 成功したらtrueを返す
	 */
	virtual bool restoreGraphicsDeviceObjects();

	/**
	 * デバイスオブジェクトの無効化
	 *
	 * デバイスオブジェクトの無効化はinvalidateDeviceObjectsをオーバライドする
	 */
	virtual void invalidateGraphicsDeviceObjects();

protected:
	//--------------------------------------------------------------------------
	// オーバーライドメソッド
	//--------------------------------------------------------------------------
	/**
	 * メインループ
	 */
	virtual void mainLoop();

	/**
	 * 情報描画
	 */
	virtual void drawInformation();

	//--------------------------------------------------------------------------
	// ユーティリティメソッド
	//--------------------------------------------------------------------------
	/**
	 * 情報文字列描画
	 * @param message 描画するメッセージ
	 * @param color 文字の描画色
	 * @param alignRight trueなら右に寄せて描画する
	 * @param alignBottom trueなら下に寄せて描画する
	 */
	virtual void drawInformationString(const String& message,
		Color4c color = Color4c::white,
		bool alignRight = false, bool alignBottom = false);

	/**
	 * FPS文字列の取得
	 * @return FPS文字列
	 */
	virtual String getFPSString() const;

	//--------------------------------------------------------------------------
	// フレームワークメソッド
	//--------------------------------------------------------------------------
	/**
	 * フレームワーク実行
	 */
	virtual void frameworkRun();

	/**
	 * フレームワーク情報表示
	 */
	virtual void frameworkDrawInformation();

	//--------------------------------------------------------------------------
	/// 情報描画矩形
	RECT informationDrawRect_;
	/// フォント
	ID3DXFont* font_;
	/// フォント記述
	D3DXFONT_DESC fontDescription_;

	/// 描画FPS計測
	FPSMeasurement* drawFPSMeasurement_;
	/// 描画FPS
	float drawFPS_;
	/// ゲームFPS計測
	FPSMeasurement* gameFPSMeasurement_;
	/// ゲームFPS
	float gameFPS_;
};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of BASIC_FRAMEWORK_H_
//------------------------------------------------------------------------------

